/***

lrfildes.c -- Local Resources, file descriptors

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

static const char*rcsid =
       "$Id: lrfildes.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#define LR_NO_FILDES_TRACKING 1
#include "lrfildes.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "lrtypes.h"
#include "lrerrs.h"

EHFILENAME(lrfildes.c);

/***
_lrCreat()
Track creat() calls.
***/

int
_lrCreat(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   const char *pcPath,
   mode_t mode)

{
   volatile int iFilDes = -1;

   EHTRY(_lrCreat) {

      iFilDes = creat(pcPath, mode);
      if (iFilDes == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_CREAT);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, pcPath,
            (pfv_ulv)close, 1L, 
            (ulong)iFilDes);

   } EHCATCH;

   /* recovery */
   if (ehError() && iFilDes != -1)
   {
      close(iFilDes);
      EHRETURN(-1);
   }

   EHRETURN(iFilDes);
} /* _lrCreat */


/***
_lrOpen()
Track open() calls.
***/

int
_lrOpen(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   const char *pcPath,
   int iOFlag)
{
   volatile int iFilDes = -1;

   EHTRY(_lrOpen) {

      iFilDes = open(pcPath, iOFlag);
      if (iFilDes == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_OPEN);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, pcPath,
            (pfv_ulv)close, 1L, 
            (ulong)iFilDes);

   } EHCATCH;

   /* recovery */
   if (ehError() && iFilDes != -1)
   {
      close(iFilDes);
      EHRETURN(-1);
   }

   EHRETURN(iFilDes);
} /* _lrOpen */


/***
_lrDup()
Track dup() calls.
***/

int
_lrDup(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int iFilDesOrg)
{
   volatile int iFilDesNew = -1;

   EHTRY(_lrDup) {

      iFilDesNew = dup(iFilDesOrg);
      if (iFilDesNew == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_DUP);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, "duped file descriptor",
            (pfv_ulv)close, 1L, 
            (ulong)iFilDesNew);

   } EHCATCH;

   /* recovery */
   if (ehError() && iFilDesNew != -1)
   {
      close(iFilDesNew);
      EHRETURN(-1);
   }

   EHRETURN(iFilDesNew);
} /* _lrDup */


/***
_lrClose()
Track close() calls.
***/

void
_lrClose(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int iFilDes)
{
   EHTRY(_lrClose) {} EHCATCH; /* no body but we need a clean up section */

   /* mark the resource and release it */
   ehResRel(pcFile,pcFunc,iLine,&ehResTrk,
         (int)LR_RES_FILDES, (pfv_ulv)close, 1L, 
         (ulong)iFilDes);

   EHLEAVE();
} /* _lrClose */


