/***

lrfildes.c -- Local Resources, file descriptors

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

static const char*rcsid =
       "$Id: lrfildes.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#define LR_NO_FILDES_TRACKING 1
#include "lrfildes.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "lrinit.h"
#include "lrerrs.h"

EHFILENAME(lrfildes.c);

/***
_lrCreat()
Track creat() calls.
***/

int
_lrCreat(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   const char *pcPath,
   mode_t mode)

{
   volatile int iFilDes = -1;

   EHTRY(_lrCreat) {

      iFilDes = creat(pcPath, mode);
      if (iFilDes == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_CREAT);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, pcPath,
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)iFilDes);

   } EHCATCH;

   /* recovery: if an error occurred after having created the file */
   /* close the file descriptor and return an error code */
   if (ehError() && iFilDes != -1)
   {
      close(iFilDes);
      EHRETURN(-1);
   }

   EHRETURN(iFilDes);
} /* _lrCreat */


/***
_lrOpen()
Track open() calls.
***/

int
_lrOpen(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   const char *pcPath,
   int iOFlag)
{
   volatile int iFilDes = -1;

   EHTRY(_lrOpen) {

      if (iOFlag & O_CREAT)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_OCREAT);
         break;
      }

      iFilDes = open(pcPath, iOFlag);
      if (iFilDes == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_OPEN);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, pcPath,
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)iFilDes);

   } EHCATCH;

   /* recovery: if an error occurred after having opened the file */
   /* close the file and return an error code */
   if (ehError() && iFilDes != -1)
   {
      close(iFilDes);
      EHRETURN(-1);
   }

   EHRETURN(iFilDes);
} /* _lrOpen */


/***
_lrDup()
Track dup() calls.
dup() returns a file descriptor which is a 'duplicate' of (that is, 
it refers to the same file as) the file descriptor argument.  The 
value of the returned file descriptor will be the lowest available 
file descriptor number.  dup() will return an error (-1) if the 
file descriptor argument is not open or it is an invalid file 
descriptor.
***/

int
_lrDup(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int iFilDesOrg)
{
   volatile int iFilDesNew = -1;

   EHTRY(_lrDup) {

      iFilDesNew = dup(iFilDesOrg);
      if (iFilDesNew == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_DUP);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, "duped file descriptor",
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)iFilDesNew);

   } EHCATCH;

   /* recovery: if an error occurred after having dup'ed the file */
   /* close the new file descriptor and return an error code */
   if (ehError() && iFilDesNew != -1)
   {
      close(iFilDesNew);
      EHRETURN(-1);
   }

   EHRETURN(iFilDesNew);
} /* _lrDup */


/***
_lrDup2()
Track dup2() calls.
dup2() returns a file descriptor which is a 'duplicate' of (that is, 
it refers to the same file as) the first file descriptor argument.  
The value of the returned file descriptor will be the same as (or 
greater than) that of the second file descriptor argument.  The file 
(currently) associated with the second file descriptor is first 
closed (if it is open).  dup2() will return an error (-1) if the 
second file descriptor argument is an invalid file descriptor.
If the first and second file descriptors are equal and valid,
that file descriptor will be returned without being closed.
***/

int 
_lrDup2(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int iFilDes,  /* file descriptor to duplicate (high_fildes) */
   int iFilDes2) /* file descriptor to close/return */
{
   volatile int iFilDesNew = -1;
   volatile bool reobtained = false;

   EHTRY(_lrDup2)
   {

      /* attempt to dup2() the file descriptor */
      /* we'll let the call itself tell us if its */
      /* valid or not */
      iFilDesNew = dup2(iFilDes, iFilDes2);
      if (iFilDesNew == -1)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_ERROR, LR_DUP2);
         break;
      }

      /***

         OK, the call was successful.  This could either mean that 
         a new file number was returned, or if iFilDes == iFilDes2,
         then iFilDesNew == iFilDes == iFilDes2 and the same file
         descriptor was returned.  In the former case (a new file 
         number was returned), iFilDes2 was closed and we need to 
         mark that in the resource tracking tables.  In the latter 
         case (the same file number was returned), iFilDes2 was not 
         closed and we don't need to mark that fact.

      ***/

      /* check the easy case first */
      if (iFilDesNew == iFilDes && iFilDes == iFilDes2)
      {
         /* verify that the user is operating on an open fd */
         if (!ehResVerify(&ehResTrk, (ulong)iFilDes, (int)LR_RES_FILDES))
         {
            ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_DUP2NOTOPEN);
            break;
         }
      }
      else if (iFilDesNew != iFilDes) /* a more typical scenario */
      {
         /***

            If a file descriptor was returned which has a value 
            different from iFilDes, there are a couple of 
            possibilities.  First, which is typically the case, 
            iFilDes == iFilDes2. That is, iFilDes2 was first closed,
            then attached to the same file as iFilDes.
            
            Second, iFilDes != iFilDes2.  In this case, iFilDes2 was
            closed (supposedly), but (for perhaps transient reasons) a 
            different file descriptor was ultimately attached to the 
            same file as iFilDes.
            
            Both of these possibilites are handled by ehResReObt(), 
            which handles the situation of iFilDes2 being "re-obtained" 
            as iFilDesNew.

            However, since iFilDes2 could be stdxxx, it might not be in 
            the resource tables because it was not explicitly opened by 
            the user.  Before we call ehResReObt() we must therefore 
            make sure that iFilDes2 is already in the tracking tables 
            by calling ehResVerify().  If it is not in the tracking 
            tables, then we only need to call ehResObt(iFilDesNew) 
            instead, since we are only now explicitly obtaining 
            iFilDesNew for the first time.

         ***/

         /* it's in the tables, so re-obtain it */
         reobtained = ehResReObt(pcFile,pcFunc,iLine,&ehResTrk,
               (int)LR_RES_FILDES, "dup2ed file descriptor",
               (ulong)iFilDes2,    /* resource that was released */
               (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
               (ulong)iFilDesNew);

      }
      else /* iFilDesNew == iFilDes && iFilDes != iFilDes2 */
      {
         /***
         I don't understand how this is possible since what we are 
         trying to do is duplicate iFilDes, that is generate a 
         *different* file descriptor that refers to the *same* file, 
         preferably having a value (of at least) iFilDes2.
         ***/
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_DUP2INTERNAL);
         break;
      }
 
   } EHCATCH;

   /***
   recovery: if an error occurred after having successfully dup'ed 
   the first file descriptor with the value of the second file 
   descriptor (or greater), then we need to try to undo what was 
   done.  This is only possible to a limited extent.  iFilDes2 was 
   (probably) closed and we can't un-close it.  However, iFilDesNew 
   is a duplication of iFilDes, and it can be unduplicated by 
   being closed.  Also we need to try to unregister resources that
   were obtained but no longer are.
   ***/

   if (ehError() && iFilDesNew != -1)
   {
      if (reobtained)
      {
         /* mark iFilDesNew as closed */
         ehResRel(pcFile,pcFunc,iLine,&ehResTrk,
               (int)LR_RES_FILDES, (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L, 
               (ulong)iFilDesNew);
      }
      /* if not "reobtained" just close iFilDesNew */
      if (iFilDesNew != iFilDes) /* exclude the trivial scenario */
      {
         close(iFilDesNew); /* close the new file */
         /* now both files are closed */
      }
      EHRETURN(-1);
   } /* ehError() */

   EHRETURN(iFilDes2);
} /* _lrDup2 */


/***
_lrClose()
Track close() calls.
***/

void
_lrClose(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int iFilDes)
{
   EHTRY(_lrClose) {} EHCATCH; /* no body but we need a clean up section */

   /* mark the resource as released (and release it) */
   ehResRel(pcFile,pcFunc,iLine,&ehResTrk,
         (int)LR_RES_FILDES, (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
         (ulong)iFilDes);

   EHLEAVE();
} /* _lrClose */


/***
_lrFileno()
Track fileno() calls.
Returns a file descriptor number.
***/

int
_lrFileno(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   FILE *pFile)
{
   int iFilDes = -1;

   EHTRACE(_lrFileno);

   /* verify that the file stream is open */
   if (!ehResVerify(&ehResTrk, (ulong)pFile, (int)EH_RES_FILESTR))
   {
      ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_BADFILESTR);
      EHRETURN(-1);
   }

   /* get the file descriptor */
   iFilDes = fileno(pFile);
   if (iFilDes == -1)
   {
      ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_FILENO);
      EHRETURN(-1);
   }

   /* it is possible that the returned file descriptor is */
   /* already open, fileno(stderr) for example.  Before */
   /* marking it as open, make sure it is not already open */
   if (!ehResVerify(&ehResTrk, (ulong)iFilDes, (int)LR_RES_FILDES))
   {
      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, "fileno'd file desc",
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)iFilDes);
   }
   else
      EHWRN1("file descriptor 0x%x already obtained", iFilDes);

   /* on release of pFile, also release iFilDes */
   ehResOnRel(&ehResTrk, (ulong)pFile,   (int)EH_RES_FILESTR,
                         (ulong)iFilDes, (int)LR_RES_FILDES);

   EHRETURN(iFilDes);
} /* _lrFileno */


/***
_lrPipe()
Track pipe() calls.
***/

int
_lrPipe(
   const char *pcFile,
   const char *pcFunc,
   int iLine,
   int *piFilDes)
{
   volatile int iRetCode = -1; /* assume an error */
   volatile bool obtained = false;

   EHTRY(_lrPipe) {

      iRetCode = pipe(piFilDes); /* get the two ends of the pipe */
      if (iRetCode != 0)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_PIPE);
         break;
      }

      obtained = ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)LR_RES_FILDES, "pipe[0]",
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)piFilDes[0]);

      /* -iLine prevents the pass count from being incremented twice */
      ehResObt(pcFile,pcFunc,-iLine,&ehResTrk,
            (int)LR_RES_FILDES, "pipe[1]",
            (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)piFilDes[1]);

   } EHCATCH;

   /* recovery */
   if (ehError() && iRetCode == 0) /* resource was obtained */
   {
      /***
      if the second ehResObt() call fails,
      it will clean up after itself.
      But we still need to clean up after the first call.
      ***/
      close(piFilDes[1]); /* resource still needs releasing */
      if (obtained)       /* resource was registered as obtained */
      {
         /* unregister it */
         ehResRel(__FILE__,"_lrPipe",__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, (pfv_ulv)close, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)piFilDes[0]);
      }
      else
      {
         /* otherwise, the actual resource was obtained, */
         /* but it didn't get marked as obtained, so just close it */
         close(piFilDes[0]);
      }
      EHRETURN(-1);
   } /* ehError() */

   EHRETURN(iRetCode);
} /* _lrPipe */


