/***

t_filtrk.c -- rigorous test driver for file stream tracking routines

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#include <string.h>

#include "lrfiltrk.h"
#include "lrfildes.h"

static void fopen_and_fclose();
static void fopen_freopen_and_fclose();
static void fopen_freopen_and_no_fclose();
static void fopen_and_no_fclose();

static void tmpfile_and_fclose();
static void tmpfile_freopen_and_fclose();
static void tmpfile_freopen_and_no_fclose();
static void tmpfile_and_no_fclose();

static void fdopen_and_fclose();
static void fdopen_freopen_and_fclose();
static void fdopen_freopen_and_no_fclose();
static void fdopen_and_no_fclose();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv) {

      fopen_and_fclose();
      fopen_freopen_and_fclose();
      fopen_freopen_and_no_fclose();
      fopen_and_no_fclose();

      tmpfile_and_fclose();
      tmpfile_freopen_and_fclose();
      tmpfile_freopen_and_no_fclose();
      tmpfile_and_no_fclose();

      fdopen_and_fclose();
      fdopen_freopen_and_fclose();
      fdopen_freopen_and_no_fclose();
      fdopen_and_no_fclose();

   } EHENDMAIN;

   EHDONE(ehError());
}


static
void
fopen_and_fclose()
{
   FILE*volatile pFile = NULL;

   EHTRY(fopen_and_fclose) {

      pFile = fopen("t_filtrk.1", "w");

   } EHCATCH;

   if (pFile) fclose(pFile);
}


static
void
fopen_freopen_and_fclose()
{
   /* for completely robust recovery, use two file pointers */
   FILE *volatile pFile1 = NULL;
   FILE *volatile pFile2 = NULL;

   EHTRY(fopen_freopen_and_fclose) {

      pFile1 = fopen("t_filtrk.2", "w");
      pFile2 = freopen("t_filtrk.3", "w", pFile1);

   } EHCATCH;

   if (pFile2)
      fclose(pFile2);
   else if (pFile1 && !ehError())
      fclose(pFile1);
}


static
void
fopen_freopen_and_no_fclose()
{
   FILE *pFile = NULL;
   EHTRACE(fopen_freopen_and_no_fclose);
   pFile = fopen("t_filtrk.4", "w");
   pFile = freopen("t_filtrk.5", "w", pFile);
}


static
void
fopen_and_no_fclose()
{
   FILE *pFile = NULL;
   EHTRACE(fopen_and_no_fclose);
   pFile = fopen("t_filtrk.6", "w");
}


static
void
tmpfile_and_fclose()
{
   FILE*volatile pFile = NULL;

   EHTRY(tmpfile_and_fclose) {

      pFile = tmpfile();

   } EHCATCH;

   if (pFile) fclose(pFile);
}


static
void
tmpfile_freopen_and_fclose()
{
   FILE *volatile pFile1 = NULL;
   FILE *volatile pFile2 = NULL;

   EHTRY(tmpfile_freopen_and_fclose) {

      pFile1 = tmpfile();
      pFile2 = freopen("t_filtrk.7", "w", pFile1);

   } EHCATCH;

   if (pFile2)
      fclose(pFile2);
   else if (pFile1 && !ehError())
      fclose(pFile1);
}


static
void
tmpfile_freopen_and_no_fclose()
{
   FILE *pFile = NULL;
   EHTRACE(tmpfile_freopen_and_no_fclose);
   pFile = tmpfile();
   pFile = freopen("t_filtrk.8", "w", pFile);
}


static
void
tmpfile_and_no_fclose()
{
   FILE *pFile = NULL;
   EHTRACE(tmpfile_and_no_fclose);
   pFile = tmpfile();
}


#include <sys/stat.h> /* for S_IXXX symbols */


static
void
fdopen_and_fclose()
{
   volatile int iFilDes = -1;
   FILE*volatile pFile = NULL;

   EHTRY(fdopen_and_fclose) {

      iFilDes = creat("t_filtrk.9", S_IWUSR);
      pFile = fdopen(iFilDes, "w");

   } EHCATCH;

   if (pFile) fclose(pFile);
   if (iFilDes != -1) close(iFilDes);
}


static
void
fdopen_freopen_and_fclose()
{
   volatile int iFilDes = -1;
   FILE *volatile pFile1 = NULL;
   FILE *volatile pFile2 = NULL;

   EHTRY(fopen_freopen_and_fclose) {

      iFilDes = creat("t_filtrk.A", S_IWUSR);
      pFile1 = fdopen(iFilDes, "w");
      pFile2 = freopen("t_filtrk.B", "w", pFile1);

   } EHCATCH;

   if (pFile2)
      fclose(pFile2);
   else if (pFile1 && !ehError())
      fclose(pFile1);
   if (iFilDes != -1) close(iFilDes);
}


static void fdopen_freopen_and_no_fclose();
static
void
fdopen_freopen_and_no_fclose()
{
   int iFilDes = -1;
   FILE *pFile = NULL;

   EHTRY(fdopen_freopen_and_no_fclose) {

      iFilDes = creat("t_filtrk.D", S_IWUSR);
      pFile = fdopen(iFilDes, "w");
      pFile = freopen("t_filtrk.E", "w", pFile);

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static void fdopen_and_no_fclose();
static
void
fdopen_and_no_fclose()
{
   int iFilDes = -1;
   FILE *pFile = NULL;

   EHTRY(fdopen_and_no_fclose) {

      iFilDes = creat("t_filtrk.F", S_IWUSR);
      pFile = fdopen(iFilDes, "w");

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


