/***

t_trkmem.c -- MemTrack, test memory tracking functions

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#include <string.h>

#include "lrmemtrk.h"

static void malloc_and_free();
static void malloc_realloc_and_free();
static void malloc_realloc_and_no_free();
static void malloc_and_no_free();

static void calloc_and_free();
static void calloc_realloc_and_free();
static void calloc_realloc_and_no_free();
static void calloc_and_no_free();

static void strdup_and_free();
static void strdup_realloc_and_free();
static void strdup_realloc_and_no_free();
static void strdup_and_no_free();

static void ehDynArr_and_free();
static void ehDynArr_ehDynArr_and_free();
static void ehDynArr_ehDynArr_and_no_free();
static void ehDynArr_and_no_free();

static void new_and_delete();
static void new_and_no_delete();

static void malloc_and_delete();
static void new_and_free();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv) {

      malloc_and_free();
      malloc_realloc_and_free();
      malloc_realloc_and_no_free();
      malloc_and_no_free();

      calloc_and_free();
      calloc_realloc_and_free();
      calloc_realloc_and_no_free();
      calloc_and_no_free();

      strdup_and_free();
      strdup_realloc_and_free();
      strdup_realloc_and_no_free();
      strdup_and_no_free();

      ehDynArr_and_free();
      ehDynArr_ehDynArr_and_free();
      ehDynArr_ehDynArr_and_no_free();
      ehDynArr_and_no_free();

      new_and_delete();
      new_and_no_delete();

      malloc_and_delete();
      new_and_free();

   } EHENDMAIN;

   EHDONE(ehError());
}


static
void
malloc_and_free()
{
   char *volatile pc = NULL;

   EHTRY(malloc_and_free) {

      pc = (char*)malloc(1234);

   } EHCATCH;

   if (pc) free(pc);
}


static
void
malloc_realloc_and_free()
{
   char *volatile pc = NULL;

   EHTRY(malloc_realloc_and_free) {

      pc = (char*)malloc(1234);
      pc = (char*)realloc(pc, 2345);

   } EHCATCH;

   if (pc) free(pc);
}


static
void
malloc_realloc_and_no_free()
{
   char *volatile pc = NULL;

   EHTRACE(malloc_realloc_and_no_free);

   pc = (char*)malloc(1234);
   pc = (char*)realloc(pc, 2345);
}


static
void
malloc_and_no_free()
{
   char *volatile pc = NULL;

   EHTRACE(malloc_and_no_free);

   pc = (char*)malloc(1234);
}


static
void
calloc_and_free()
{
   char *volatile pc = NULL;

   EHTRY(calloc_and_free) {

      pc = (char*)calloc(123, 456);

   } EHCATCH;

   if (pc) free(pc);
}


static
void
calloc_realloc_and_free()
{
   char *volatile pc = NULL;

   EHTRY(calloc_realloc_and_free) {

      pc = (char*)calloc(123, 456);
      pc = (char*)realloc(pc, 123456);

   } EHCATCH;

   if (pc) free(pc);
}


static
void
calloc_realloc_and_no_free()
{
   char *volatile pc = NULL;

   EHTRACE(calloc_realloc_and_no_free);

   pc = (char*)calloc(123, 456);
   pc = (char*)realloc(pc, 123456);
}


static
void
calloc_and_no_free()
{
   char *volatile pc = NULL;

   EHTRACE(calloc_and_no_free);

   pc = (char*)calloc(123, 456);
}


static
void
strdup_and_free()
{
   char *volatile pc = NULL;

   EHTRY(strdup_and_free) {

      pc = strdup("First string to duplicate.");

   } EHCATCH;

   if (pc) free(pc);
}


static
void
strdup_realloc_and_free()
{
   char *volatile pc = NULL;

   EHTRY(strdup_realloc_and_free) {

      pc = strdup("Second string to duplicate.");
      pc = (char*)realloc(pc, 123456);

   } EHCATCH;

   if (pc) free(pc);
}


static
void
strdup_realloc_and_no_free()
{
   char *volatile pc = NULL;

   EHTRACE(strdup_realloc_and_no_free);

   pc = strdup("Third string to duplicate.");
   pc = (char*)realloc(pc, 123456);
}


static
void
strdup_and_no_free()
{
   char *pc = NULL;

   EHTRACE(strdup_and_no_free);

   pc = strdup("Forth string to duplicate.");
}


static
void
ehDynArr_and_free() /* incrementally extend a dynamic array */
{
   FILE *volatile pFileArr = NULL; /* base of the array of FILE's */

   EHTRY(ehDynArr_and_free)
   {
      size_t szFileArr = 0;        /* current size of the array */
      FILE *pFile;                 /* pointer to array element */

      pFile =                      /* point to newest array element */
            (FILE*)ehDynArr(       /* extend the array by one element */
            (void**)&pFileArr,     /* address of base ptr to array */
            &szFileArr,            /* current # elements in array */
            sizeof(FILE));         /* size of each element */

   } EHCATCH;

   if (pFileArr) free(pFileArr);
}


static
void
ehDynArr_ehDynArr_and_free()
{
   FILE *volatile pFileArr = NULL;

   EHTRY(ehDynArr_ehDynArr_and_free)
   {
      size_t szFileArr = 0;
      FILE *pFile;

      /* get pointer to first element */
      pFile = (FILE*)ehDynArr(
            (void**)&pFileArr, &szFileArr, sizeof(FILE));
      /* get pointer to second element */
      pFile = (FILE*)ehDynArr(
            (void**)&pFileArr, &szFileArr, sizeof(FILE));

   } EHCATCH;

   if (pFileArr) free(pFileArr);
}


static
void
ehDynArr_ehDynArr_and_no_free()
{
   FILE *volatile pFileArr = NULL;
   size_t szFileArr = 0;
   FILE *pFile;

   EHTRACE(ehDynArr_ehDynArr_and_no_free);

   pFile = (FILE*)ehDynArr((void**)&pFileArr, &szFileArr, sizeof(FILE));
   pFile = (FILE*)ehDynArr((void**)&pFileArr, &szFileArr, sizeof(FILE));

}


static
void
ehDynArr_and_no_free()
{
   FILE *volatile pFileArr = NULL;
   size_t szFileArr = 0;
   FILE *pFile;

   EHTRACE(ehDynArr_and_no_free);

   pFile = (FILE*)ehDynArr((void**)&pFileArr, &szFileArr, sizeof(FILE));

}


static
void
new_and_delete()
{
   char *volatile pc = NULL;

   EHTRY(new_and_delete) {

      pc = new char[1234];

   } EHCATCH;

   if (pc) delete [] pc;
}


static
void
new_and_no_delete()
{
   char *volatile pc = NULL;

   EHTRACE(new_and_no_delete);

   pc = new char[1234];
}


static
void
malloc_and_delete()
{
   char *volatile pc = NULL;

   EHTRY(malloc_and_delete) {

      pc = (char*)malloc(1234);

   } EHCATCH;

   if (pc) delete [] pc;
   EHSTOPCHAIN();   /* stop chaining process */
   EHCLEAR();       /* clear the error state */
}


static void new_and_free();
static
void
new_and_free()
{
   char *volatile pc = NULL;

   EHTRY(new_and_free) {

      pc = new char[1234];

   } EHCATCH;

   if (pc) free(pc);
   EHSTOPCHAIN();   /* stop chaining process */
   EHCLEAR();       /* clear the error state */
}



