/***

ehdynarr.c -- Error Handler, Dynamic Array allocation

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehdynarr.c,v 1.6 1996/04/12 22:10:51 ccobb Exp $";

#include "ehdynarr.h"

EHBEGINEXTERNC
#include <stdlib.h>
#include <string.h>
EHENDEXTERNC

#include "eherrs.h"
#include "ehperror.h"

/***
ehDynArr()
Extend a dynamic array.  Returns pointer to newly created element.
WARNING: this function may reposition the array.  Typically you would only 
         maintain a SINGLE pointer to the array and a SINGLE variable
         containing the size of the array.  The address of both of these
         variables is passed to xalloc() which updates both of them.  Do 
         not use other pointers to point to this array because they would 
         become invalid if the array location were to move.  A pointer to
         the newly added element is returned by xalloc().  This pointer is
         valid only until the next call to xalloc().
***/

void *
ehDynArr A3(void**,       ppvArray, /* ADDRESS OF ptr to dynamic arr */
            size_t *,     pszNumEl, /* ptr to # el currently in arr */
            const size_t, szEl)     /* sizeof each element */
{
   char * volatile pc = NULL;
   int   iNewNumEl;
   int   iNum2Alloc;
   char **ppcArray = (char **)ppvArray;

   EHTRY(ehDynArr) {

      EHASSERT((*ppcArray == NULL && *pszNumEl == 0) ||
             (*ppcArray != NULL && *pszNumEl != 0));
      EHASSERT(szEl > 0);

      iNewNumEl = (int)(*pszNumEl + 1);
      iNum2Alloc = (int)ehRoundUp(iNewNumEl);   /* # of elements to allocate */

      /* first time, prime the array */
      if (*ppcArray == NULL)
      {
         if ((*ppcArray = (char *)malloc((size_t)(szEl * iNum2Alloc)))
               == NULL)
         {
            EHTHROW(EH_MALLOC);
         }
         pc = *ppcArray;
      }
      else /* extend dynamic array if necessary */
      {
         int iNumAllocated = ehRoundUp((uint)*pszNumEl);/*allocated last time*/

         if (iNum2Alloc > iNumAllocated) /* time to reallocate */
         {
            if ((*ppcArray = (char *)realloc(*ppcArray,
                  (size_t)(szEl * iNum2Alloc)))
                  == NULL)
            {
               EHTHROW(EH_REALLOC);
            }
         }
         pc = *ppcArray + (*pszNumEl * szEl);
      }
      memset(pc, 0, (size_t)szEl);
      (*pszNumEl)++;

   } EHCATCH;

   if (ehError())
   {
      if (*ppcArray) free(*ppcArray);
      *ppcArray = pc = NULL;
      *pszNumEl = 0;
      EHSTOPCHAIN();
   }
   EHRETURN( (void *) pc );
} /* ehDynArr */


/***
round up to the nearest power of two
***/

unsigned int 
ehRoundUp A1(unsigned int,uiValue)
{
   unsigned int uiRetVal = 1;

   EHTRACE(ehRoundUp);

   if (uiValue)
   {
      uiValue = (uiValue << 1) - 1;
      while ( (uiValue >>= 1) != 0)
      {
         uiRetVal *= 2;
      }
   }

   EHRETURN(uiRetVal);
}


/***

$Log: ehxalloc.c,v $
 * Revision 1.6  1996/04/12  22:10:51  ccobb
 * From Borland C++: now requires xh support.
 *
 * Revision 1.5  1996/04/11  06:46:05  ccobb
 * Compiled on Solaris with GCC.
 *
 * Revision 1.4  1995/09/11  05:18:36  ccobb
 * no change
 *
 * Revision 1.3  1995/08/16  19:50:17  ccobb
 * Compiled with C++.
 *
 * Revision 1.2  1995/04/09  08:36:15  ccobb
 * Well, it works now!  I can't remember all the changes.
 *

***/
